/*  File: analyser-AtlasGap.cc
 *  Project: Deductor
 *  Created by D. Soper on 2 December 2014
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-AtlasGap.h"

/*   other includes   */

void UserAtlasGap::initfunc()
{
  /*  Create histograms to calculate the fraction of two jet events with a gap
   *  as a function of the PTcut for the gap.
   *  We demand that there are two jets with PT > PTjet GeV
   *  one with Y < - Ygap and one with Y > + Ygap.
   *  Then there can be no jet in the gap -Ygap < Y < +Ygap
   *  with PT > PTcut 
   */
  auto bins = spacing<hist1d>::manual({50,60,70,80,90,105,120,135,150,180,
       210,240,270,300,340,380,420,460,500});
  
  UserRatio::phys(1, "Cross section ratio, 1 < Delta y < 2", bins);
  UserRatio::phys(2, "Cross section ratio, 2 < Delta y < 3", bins);
  UserRatio::phys(3, "Cross section ratio, 3 < Delta y < 4", bins);
  UserRatio::phys(4, "Cross section ratio, 4 < Delta y < 5", bins);
  UserRatio::phys(5, "Cross section ratio, 5 < Delta y < 6", bins);


  cout << "UserAtlasGap is using jet algorithm "
       << jetDef.jet_algorithm()
       << " with R = " << jetDef.R() << endl;
  
  cout << "    Gap is between jets, with PTcut = 20 GeV." << endl;

}



void UserAtlasGap::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;


  /*   Calculate the weight of the event   */
  constexpr double tonb = 389379.338;
  double wb = sb.weight*color_weight(sb)*tonb;
  
  /* preparing the momenta for FastJet */
  make_pseudojets(theparticles, p);
  
  /*   Clustering the partons using FastJet */
  ClusterSequence clustSeq(theparticles, jetDef);
  
  /*   Define pTcut */
  double pTcut = 20.0;
  /*   Extract inclusive jets (pT > pTcut) */
  incJets0 = clustSeq.inclusive_jets(pTcut);
  incJets = sorted_by_pt(incJets0);
  
  // Find two leading PT jets in allowed rapidity range.
  PseudoJet jet[2];
  unsigned int nj = 0;
  for(unsigned int n = 0; n < incJets.size() && nj < 2; n++)
    if(std::abs(incJets[n].rapidity()) < 4.4) jet[nj++] = incJets[n];
  if(nj < 2) return;

  double Ymin = jet[0].rapidity(), Ymax = jet[1].rapidity();
  double DeltaY = std::abs(Ymax - Ymin);
  if(Ymin > Ymax) std::swap(Ymin, Ymax);
  double pTaverage = 0.5*(jet[0].perp() + jet[1].perp());
  
  double tiny = 1.0e-10;
  double wg = wb; // wg is weight if nothing in gap, otherwise 0.
  
  for(unsigned int i = 0; i < incJets.size(); i++)
    if(incJets[i].pt() > pTcut &&
       incJets[i].rapidity() < Ymax - tiny &&
       incJets[i].rapidity() > Ymin + tiny )
    {
      wg = 0.0;
      break;
    }
  
  if(1.0 < DeltaY && DeltaY < 2.0)
  {
    UserRatio::physfill(1, {{wg, wb}}, dirac(), pTaverage);
  }
  
  if (2.0 < DeltaY && DeltaY < 3.0)
  {
    UserRatio::physfill(2, {{wg, wb}}, dirac(), pTaverage);
  }
  
  if(3.0 < DeltaY && DeltaY < 4.0)
  {
    UserRatio::physfill(3, {{wg, wb}}, dirac(), pTaverage);
  }
  
  
  if (4.0 < DeltaY && DeltaY < 5.0)
  {
    UserRatio::physfill(4, {{wg, wb}}, dirac(), pTaverage);
  }
  
  if(5.0 < DeltaY && DeltaY < 6.0)
  {
    UserRatio::physfill(5, {{wg, wb}}, dirac(), pTaverage);
  }
  


}

