/*  File: analyser-gap.cc
 *  Project: Deductor
 *  Created by D. Soper on 19 March 2014
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-gap.h"

/*   step function reversed */

double stepbar(const hist1d& y, double x)
  {
    if(x < y.xmin) return 1.0;
    if(x < y.xmax) return (y.xmax - x)/(y.xmax - y.xmin);
    return 0.0;
  }


/*   other includes   */

void UserGap::initfunc()
{
  /*  Create histograms to calculate the fraction of two jet events with a gap
   *  as a function of the PTcut for the gap.
   *  We demand that there are two jets with PT > PTjet GeV
   *  one with Y < - Ygap and one with Y > + Ygap.
   *  Then there can be no jet in the gap -Ygap < Y < +Ygap
   *  with PT > PTcut 
   */
  unsigned int nPTbins = 40;
  double PTmin = 1.0, PTmax = 200.0;
  auto bins = spacing<hist1d>::logarithmic(nPTbins, PTmin, PTmax);
  UserVoid::phys(1,"Cross section with or without gap, all I");
  UserPlain::phys(1, "With gap -2 < y < 2, PTjet > 200 GeV, all I", bins);
  UserVoid::phys(2,"Cross section with or without gap, I[0] = 0");
  UserPlain::phys(2, "With gap -2 < y < 2, PTjet > 200 GeV, I[0] = 0", bins);
  UserVoid::phys(3,"Cross section with or without gap, I[0] = 1");
  UserPlain::phys(3, "With gap -2 < y < 2, PTjet > 200 GeV, I[0] = 1", bins);
  UserVoid::phys(4,"Cross section with or without gap, I[0] = 2");
  UserPlain::phys(4, "With gap -2 < y < 2, PTjet > 200 GeV, I[0] = 2", bins);
  UserVoid::phys(5,"Cross section with or without gap, connected");
  UserPlain::phys(5, "With gap -2 < y < 2, PTjet > 200 GeV, connected", bins);
  UserVoid::phys(6,"Cross section with or without gap, disconnected");
  UserPlain::phys(6, "With gap -2 < y < 2, PTjet > 200 GeV, disconnected", bins);

  cout << "UserGap is using jet algorithm "
       << jetDef.jet_algorithm()
       << " with R = " << jetDef.R() << endl;
  
  cout << "    Gap is from -2 to 2, with PTjet > 200 GeV." << endl;

}



void UserGap::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  /*   the shower stage before the shower evolution */
  auto& sf = s.front();
  auto& pf = sf.state;
  int I0 = pf.color_suppression_index;


  /*   Calculate the weight of the event   */
  constexpr double tonb = 389379.338;
  double wb = sb.weight*color_weight(sb)*tonb;
  
  /* preparing the momenta for FastJet */
  make_pseudojets(theparticles, p);
  
  /*   Clustering the partons using FastJet */
  ClusterSequence clustSeq(theparticles, jetDef);
  
  /*   Extract inclusive jets (pT > 1 GeV) */
  incJets = clustSeq.inclusive_jets(1.0_GeV);
  int nJetsLeft  = 0;
  int nJetsRight = 0;
  double PTmax = 1.0;
  
  for(unsigned int n = 0; n < incJets.size(); n++)
  {
    if(incJets[n].rap() < -2.0 && incJets[n].perp() > 200.0)
      nJetsLeft++;
    if(incJets[n].rap() > 2.0  && incJets[n].perp() > 200.0)
      nJetsRight++;
    if(std::abs(incJets[n].rap()) < 2.0 && incJets[n].perp() > PTmax)
      PTmax = incJets[n].perp();
  }
  
  if (nJetsLeft > 0 && nJetsRight > 0)
  {
    bool c12  = (pf.are_color_connected(1,2)  || pf.are_colorP_connected(1,2));
    bool cab  = (pf.are_color_connected(-1,0) || pf.are_colorP_connected(-1,0));
    bool ca1  = (pf.are_color_connected(-1,1) || pf.are_colorP_connected(-1,1));
    bool ca2  = (pf.are_color_connected(-1,2) || pf.are_colorP_connected(-1,2));
    bool cb1  = (pf.are_color_connected(0,1)  || pf.are_colorP_connected(0,1));
    bool cb2  = (pf.are_color_connected(0,2)  || pf.are_colorP_connected(0,2));
    bool connected = false;
    if (c12) connected = true;
    else if (cab) connected = true;
    else if ((pf[1].momentum.rapidity() > 0) && cb1)  connected = true;
    else if ((pf[1].momentum.rapidity() < 0) && ca1)  connected = true;
    else if ((pf[2].momentum.rapidity() > 0) && cb2)  connected = true;
    else if ((pf[2].momentum.rapidity() < 0) && ca2)  connected = true;

    //
    UserVoid::physfill(1, wb);
    UserPlain::physfill(1, wb, stepbar, PTmax);
    
    if (I0 == 0)
    {
      UserVoid::physfill(2, wb);
      UserPlain::physfill(2, wb, stepbar, PTmax);
    }
    if (I0 == 1)
    {
      UserVoid::physfill(3, wb);
      UserPlain::physfill(3, wb, stepbar, PTmax);
    }
    if (I0 == 2)
    {
      UserVoid::physfill(4, wb);
      UserPlain::physfill(4, wb, stepbar, PTmax);
    }
    if (connected)
    {
      UserVoid::physfill(5, wb);
      UserPlain::physfill(5, wb, stepbar, PTmax);
    }
    if (!connected)
    {
        UserVoid::physfill(6, wb);
        UserPlain::physfill(6, wb, stepbar, PTmax);
    }
  }
}

